from itertools import cycle


def plotnine_titles(
    plotnine_fig,
    title=None,
    subtitle=None,
    caption=None,
    fontfamily="Inter",
    title_size=20,
    title_color="black",
    subtitle_size=15,
    subtitle_color="darkgrey",
    caption_size=10,
    caption_color="grey",
    draw_fig=True,
    **kwargs
):
    """
    Adds titles, subtitles and captions to a plotnine figure.
      For text (title, subtitle and caption) and the color and size for each element you can pass either a string or a array of strings.
      If an array of strings is used for the text variable then arrays on the other variables are used to reformat each of the strings in the text array (e.g. color part of the text a different color).

    Args:
        plotnine_fig: The Plotnine figure object.
        title (str or array, optional): The title of the plot. Defaults to None.
        subtitle (str or array, optional): The subtitle of the plot. Defaults to None.
        caption (str or array, optional): The caption of the plot. Defaults to None.
        fontfamily (str, optional): The font family to use for text elements. Defaults to None.
        title_size (int or array, optional): The font size for the title. Defaults to 32.
        title_color (str or array, optional): The color of the title text. Defaults to 'black'.
        subtitle_size (int or array, optional): The font size for the subtitle. Defaults to 20.
        subtitle_color (str or array, optional): The color of the subtitle text. Defaults to 'darkgrey'.
        caption_size (int or array, optional): The font size for the caption. Defaults to 15.
        caption_color (str or array, optional): The color of the caption text. Defaults to 'grey'.
        draw_fig (bool, optional): You can only draw() the figure once. If passing in a plotnine figure directly then use True otherwise False. Defaults to True.
        **kwargs: Additional arguments for matplotlib.text().

    Returns:
        The customized Plotnine figure object.
    """
    if draw_fig:
        plotnine_fig = plotnine_fig.draw()

    y_title = 1.02 if subtitle is None or subtitle == "" else 1.10
    if title is not None:
        plotnine_fig = plotnine_text(
            plotnine_fig=plotnine_fig,
            text=title,
            x=0,
            y=y_title,
            size=title_size,
            color=title_color,
            ha="left",
            va="bottom",
            fontfamily=fontfamily,
            draw_fig=False,
            **kwargs
        )

    if subtitle is not None:
        plotnine_fig = plotnine_text(
            plotnine_fig=plotnine_fig,
            text=subtitle,
            x=0,
            y=1.02,
            size=subtitle_size,
            color=subtitle_color,
            ha="left",
            va="bottom",
            fontfamily=fontfamily,
            draw_fig=False,
            **kwargs
        )

    if caption is not None:
        plotnine_fig = plotnine_text(
            plotnine_fig=plotnine_fig,
            text=caption,
            x=0,
            y=-0.05,
            size=caption_size,
            color=caption_color,
            ha="left",
            va="top",
            fontfamily=fontfamily,
            draw_fig=False,
            **kwargs
        )

    return plotnine_fig


def plotnine_coloured_axis_labels(
    plotnine_fig,
    label_color_dict,
    default_label_color="#36454f",
    axis="X",
    draw_fig=True,
):
    """
    Applies custom colors to axis labels of a Plotnine figure.

    Args:
        plotnine_fig (plotnine.ggplot): The Plotnine figure to which the colored axis labels will be applied.
        label_color_dict (dict): A dictionary mapping axis labels to their corresponding colors.
        default_label_color (str, optional): The default color for labels not specified in `label_color_dict`. Default is '#36454f'.
        axis (str, optional): The axis to which the label colors will be applied. Options are 'X' or 'Y'. Default is 'X'.
        draw_fig (bool, optional): You can only draw() the figure once. If passing in a plotnine figure directly then use True otherwise False. Defaults to True.

    Returns:
        plotnine.ggplot: The Plotnine figure with the colored axis labels.
    """

    if draw_fig:
        plotnine_fig = plotnine_fig.draw()

    for ax in plotnine_fig.axes:
        if "X" in axis.upper():
            for l in ax.get_xticklabels():
                c = label_color_dict.get(l.get_text())
                if c is None:
                    c = default_label_color
                l.set_color(c)

        if "Y" in axis.upper():
            for l in ax.get_yticklabels():
                c = label_color_dict.get(l.get_text())
                if c is None:
                    c = default_label_color
                l.set_color(c)

    return plotnine_fig


def plotnine_text(
    plotnine_fig,
    x,
    y,
    text,
    color="black",
    weight="regular",
    size=15,
    style="normal",
    va="bottom",
    ax_id=0,
    draw_fig=True,
    fontfamily="Inter",
    **kwargs
):
    """
    Adds text annotations to a Plotnine figure.

    For text, color, weight, size, style you can pass either a string or a array of strings.
    If an array of strings is used for the text variable then arrays on the other variables are used to reformat each of the strings in the text array (e.g. color part of the text a different color).

    Args:
        plotnine_fig (plotnine.ggplot): The Plotnine figure to which the text will be added.
        x (float): The x-coordinate for the text position relative to the overall figure.
        y (float): The y-coordinate for the text position relative to the overall figure.
        text (str or array): The text string to be added to the plot.
        color (str or array, optional): The color of the text. Default is 'black'.
        weight (str or array, optional): The font weight of the text. Options are 'light', 'normal', 'bold', etc. Default is 'regular'.
        size (int or array, optional): The size of the text in points. Default is 20.
        style (str or array, optional): The font style of the text. Options are 'normal', 'italic', etc. Default is 'normal'.
        va (str, optional): The vertical alignment of the text. Options are 'top', 'bottom', 'center', etc. Default is 'bottom'.
        ax_id (int, optional): The index of the axis on which to draw the text (if multiple axes are present). Default is 0.
        draw_fig (bool, optional): You can only draw() the figure once. If passing in a plotnine figure directly then use True otherwise False. Default is True.
        **kwargs: Additional arguments for matplotlib.text().

    Returns:
        plotnine.ggplot: The Plotnine figure with the added text annotation.
    """
    if draw_fig:
        plotnine_fig = plotnine_fig.draw()

    ax = plotnine_fig.axes[ax_id]

    # convert to lists if not already so can iterate through
    text = text if isinstance(text, list) else [text]
    color = color if isinstance(color, list) else [color]
    weight = weight if isinstance(weight, list) else [weight]
    size = size if isinstance(size, list) else [size]
    style = style if isinstance(style, list) else [style]

    for idx, (t, c, w, sz, st) in enumerate(
        zip(text, cycle(color), cycle(weight), cycle(size), cycle(style))
    ):
        if idx == 0:
            text = plotnine_fig.text(
                s=t,
                x=x,
                y=y,
                fontweight=w,
                size=sz,
                color=c,
                style=st,
                va=va,
                fontfamily=fontfamily,
                **kwargs,
            )
        else:
            text = ax.annotate(
                text=t,
                xycoords=text,
                xy=(1, 0),
                fontweight=w,
                size=sz,
                color=c,
                style=st,
                va=va,
                fontfamily=fontfamily,
                **kwargs
            )

    return plotnine_fig



import shutil
import importlib_resources
from pathlib import Path


def import_quarto_template(
    qmd_filename: str = "template.qmd", destination_dir: str = None
):
    """
    Imports the brand themed qmd template into the working directory

    Args:
        qmd_filename (str, optional): The name of the qmd file to be imported alongside the template. Defaults to template.qmd
        destination_dir (str, optional): The directory to copy the template to. Defaults to the current working directory.

    Returns:
        None
    """
    package_name = f"{__package__}".split(".")[0]

    exclude_filenames = ["template.qmd"]

    if qmd_filename[-4:] != ".qmd":
        qmd_filename = qmd_filename + ".qmd"

    # Use importlib_resources to get the absolute path of the resource directory
    resource_dir = importlib_resources.files(package_name).joinpath("template_files")

    # Determine the destination directory
    if destination_dir is None:
        destination_dir = Path.cwd()
    else:
        if Path(destination_dir).exists() == False:
            raise FileNotFoundError(
                f"The folder {destination_dir} does not exist. Please create it and run this command again."
            )
        destination_dir = Path(destination_dir)

    # copy the template qmd file to the main directory
    if destination_dir.joinpath(qmd_filename).exists():
        print(
            f"A filename {qmd_filename} already exists and the blank qmd template will not be automatically copied."
        )
    else:
        shutil.copy2(
            resource_dir.joinpath("template.qmd"),
            destination_dir.joinpath(qmd_filename),
        )

    # create directory to copy the template files to
    quarto_dir = destination_dir.joinpath("_extensions/brandtemplate")

    if quarto_dir.exists() == False:
        quarto_dir.mkdir(parents=True)

    # Walk through the directory structure and copy files
    for src_file_path in resource_dir.glob("**/*"):
        if src_file_path.is_file() and src_file_path.name not in exclude_filenames:
            relative_path = src_file_path.relative_to(resource_dir)
            dest_file_path = quarto_dir.joinpath(relative_path)
            dest_file_path.parent.mkdir(parents=True, exist_ok=True)

            # Copy the file to the destination
            shutil.copy2(src_file_path, dest_file_path)



from .plotnine.plotnine_theme import brandtemplate_plotnine_theme
from .plotnine.plotnine_extra import (
    plotnine_titles,
    plotnine_text,
    plotnine_coloured_axis_labels,
)

from .qmd_template.import_template import import_quarto_template
from .qmd_template.cli_import_template import _cli_import_template

__all__ = [
    "brandtemplate_plotnine_theme",
    "plotnine_titles",
    "plotnine_text",
    "plotnine_coloured_axis_labels",
    "import_quarto_template",
]



from .import_template import import_quarto_template

import argparse


def _cli_import_template():
    """
    Function for running import_template via command line.
    Use import_template instead if running from python.
    """
    parser = argparse.ArgumentParser(
        description="Import the branded quarto template to your project."
    )

    parser.add_argument(
        "-n",
        "--name",
        type=str,
        default="template.qmd",
        help='The name of the qmd file to create. Defaults to "template.qmd" if not specified.',
    )
    parser.add_argument(
        "-d",
        "--directory",
        type=str,
        default=None,
        help="The directory to import the template into. Defaults to the current directory if not specified.",
    )

    args = parser.parse_args()

    import_quarto_template(qmd_filename=args.name, destination_dir=args.directory)


if __name__ == "__main__":
    _cli_import_template()



from plotnine import theme_bw, theme

from plotnine.themes.elements import (
    element_blank,
    element_line,
    element_rect,
    element_text,
)

BASE_GRAPH_COLOR = "#36454f"
GRIDLINES_COLOR = "#ededed"
TITLE_COLOR = "#36454f"
SUBTITLE_COLOR = "#5E6A72"
CAPTION_COLOR = "#5E6A72"
LEGEND_POSITION = "bottom"
LEGEND_DIRECTION = "horizontal"


class brandtemplate_plotnine_theme(theme_bw):
    """
    A custom theme for plotnine.

    Args:
        base_size (int, optional): The base font size for the plot. Defaults to 15.
        base_family (str, optional): The base font family for the plot. Defaults to 'Inter'.
        gridlines (str, optional): Determines which gridlines to display. Values are 'X', 'Y' or 'XY'. Defaults to 'X'.
        axis_title (str, optional): Determins the title for the axes to display. Values are 'X', 'Y' or 'XY'. Defaults to ''.
        axis_line (str, optional): Determines if axis lines are displayed. Values are 'X', 'Y' or 'XY'. Defaults to 'X'.
        axis_start_zero (str, optional): Recolors the axis if it doesn't start at zero. Values are 'X', 'Y' or 'XY'. Defaults to 'X'.
        title_size (int, optional): Font size for the plot title. Defaults to 32.
        subtitle_size (int, optional): Font size for the plot subtitle. Defaults to 20.
        caption_size (int, optional): Font size for the plot caption. Defaults to 15.

    Returns:
        None
    """

    def __init__(
        self,
        base_size=15,
        base_family="Inter",
        gridlines="Y",
        axis_title="",
        axis_line="X",
        axis_start_zero="X",
        title_size=32,
        subtitle_size=20,
        caption_size=10,
    ):
        super().__init__(base_size, base_family)
        self += theme(
            line=element_rect(),
            rect=element_rect(fill="white", color="black"),
            text=element_text(color=BASE_GRAPH_COLOR, fontweight="regular"),
            title=element_text(color=BASE_GRAPH_COLOR),
            plot_background=element_rect(color="white", fill="white"),
            strip_background=element_rect(fill="white"),
            plot_title=element_text(
                color=TITLE_COLOR, fontweight="bold", size=title_size
            ),
            plot_subtitle=element_text(
                color=SUBTITLE_COLOR, fontweight="regular", size=subtitle_size
            ),
            plot_caption=element_text(
                color=CAPTION_COLOR, fontweight="regular", size=caption_size
            ),
            plot_margin=0.005,
            panel_border=element_blank(),
            panel_grid_major=element_line(color=GRIDLINES_COLOR),
            panel_grid_major_x=(
                element_line(color=GRIDLINES_COLOR)
                if "X" in gridlines.upper()
                else element_blank()
            ),
            panel_grid_major_y=(
                element_line(color=GRIDLINES_COLOR)
                if "Y" in gridlines.upper()
                else element_blank()
            ),
            panel_grid_minor=element_blank(),
            axis_title_x=(
                element_text(margin={"t": 10})
                if "X" in axis_title.upper()
                else element_blank()
            ),
            axis_title_y=(
                element_text(margin={"r": 10})
                if "Y" in axis_title.upper()
                else element_blank()
            ),
            axis_ticks=element_line(color="white"),
            axis_ticks_length=8,  # used to add space between axis and text
            axis_line_x=(
                element_blank()
                if "X" not in axis_line.upper()
                else (
                    element_line()
                    if "X" in axis_start_zero.upper()
                    else element_line(color=GRIDLINES_COLOR)
                )
            ),
            axis_line_y=(
                element_blank()
                if "Y" not in axis_line.upper()
                else (
                    element_line()
                    if "Y" in axis_start_zero.upper()
                    else element_line(color=GRIDLINES_COLOR)
                )
            ),
            legend_background=element_blank(),
            legend_position=LEGEND_POSITION,
            legend_box="vertical",
            legend_direction=LEGEND_DIRECTION,
        )
